@@ -9,13 +9,20 @@ import java.util.LinkedList;
  * influence the sequence in which nodes of the search tree are evaluated, or
  * to supply a heuristic solution. To find out why the callback is issued
  * use method {@link GLPK#glp_ios_reason(glp_tree) GLPK.glp_ios_reason}.
+ * <p>The list of listeners is thread local. Each thread has to register its
+ * own listener.
  */
 public final class GlpkCallback {
     /**
      * List of callback listeners.
      */
-    private static LinkedList<GlpkCallbackListener> listeners
-            = new LinkedList<GlpkCallbackListener>();
+    private static ThreadLocal<LinkedList<GlpkCallbackListener>> listeners
+            = new ThreadLocal<LinkedList<GlpkCallbackListener>>() {
+                @Override
+                protected LinkedList<GlpkCallbackListener> initialValue() {
+                   return new LinkedList<GlpkCallbackListener>();
+                }
+            };
 
     /**
      * Constructor.
@@ -30,7 +37,7 @@ public final class GlpkCallback {
     public static void callback(final long cPtr) {
         glp_tree tree;
         tree = new glp_tree(cPtr, false);
-        for (GlpkCallbackListener listener : listeners) {
+        for (GlpkCallbackListener listener : listeners.get()) {
             listener.callback(tree);
         }
     }
@@ -40,7 +47,7 @@ public final class GlpkCallback {
      * @param listener listener for callbacks
      */
     public static void addListener(final GlpkCallbackListener listener) {
-        listeners.add(listener);
+        listeners.get().add(listener);
     }
 
     /**
@@ -49,7 +56,7 @@ public final class GlpkCallback {
      * @return true if the listener was found
      */
     public static boolean removeListener(final GlpkCallbackListener listener) {
-        return listeners.remove(listener);
+        return listeners.get().remove(listener);
     }
 
 }
