@@ -8,6 +8,8 @@ import java.util.LinkedList;
  * terminal output. A listener can inhibit the terminal output by returning
  * <code>false</code> in the {@link GlpkTerminalListener#output(String) output}
  * routine.
+ * <p>The list of listeners is thread local. Each thread has to register its
+ * own listener.
  * <p>If a {@link GlpkException GlpkExeption} has occured it is necessary to
  * call <pre>
  * GLPK.glp_term_hook(null, null);</pre>
@@ -21,8 +23,13 @@ public final class GlpkTerminal {
     /**
      * List of listeners.
      */
-    private static LinkedList<GlpkTerminalListener> listeners
-            = new LinkedList<GlpkTerminalListener>();
+    private static ThreadLocal<LinkedList<GlpkTerminalListener>> listeners
+            = new ThreadLocal<LinkedList<GlpkTerminalListener>>() {
+                @Override
+                protected LinkedList<GlpkTerminalListener> initialValue() {
+                    return new LinkedList<GlpkTerminalListener>(); 
+                }
+            };
 
     static {
         GLPK.glp_term_hook(null, null);
@@ -36,15 +43,15 @@ public final class GlpkTerminal {
 
     /**
      * Callback function called by native library.
-     * Output to the console is created if any of the listeners requests it.
+     * Output to the console is created if any of the listeners.get() requests it.
      * @param str string to be written to console
      * @return 0 if output is requested
      */
     public static int callback(final String str) {
         boolean output = false;
 
-        if (listeners.size() > 0) {
-            for (GlpkTerminalListener listener : listeners) {
+        if (listeners.get().size() > 0) {
+            for (GlpkTerminalListener listener : listeners.get()) {
                 output |= listener.output(str);
             }
             if (output) {
@@ -61,7 +68,7 @@ public final class GlpkTerminal {
      * @param listener listener for terminal output
      */
     public static void addListener(final GlpkTerminalListener listener) {
-        listeners.add(listener);
+        listeners.get().add(listener);
     }
 
     /**
@@ -70,15 +77,15 @@ public final class GlpkTerminal {
      * @return true if listener was found
      */
     public static boolean removeListener(final GlpkTerminalListener listener) {
-        return listeners.remove(listener);
+        return listeners.get().remove(listener);
     }
 
     /**
-     * Remove all listeners.
+     * Remove all listeners.get().
      */
     public static void removeAllListeners() {
-        while (listeners.size() > 0) {
-            listeners.removeLast();
+        while (listeners.get().size() > 0) {
+            listeners.get().removeLast();
         }
     }
 }
